@@ -5,37 +5,26 @@ from typing import Any, Dict, Optional
 from mypy.nodes import (
     MypyFile, SymbolNode, SymbolTable, SymbolTableNode,
     TypeInfo, FuncDef, OverloadedFuncDef, Decorator, Var,
-    TypeVarExpr, ClassDef, Block,
-    LDEF, MDEF, GDEF, TYPE_ALIAS
+    TypeVarExpr, ClassDef, Block, TypeAlias,
 )
 from mypy.types import (
-    CallableType, EllipsisType, Instance, Overloaded, TupleType, TypedDictType,
-    TypeList, TypeVarType, UnboundType, UnionType, TypeVisitor,
+    CallableType, Instance, Overloaded, TupleType, TypedDictType,
+    TypeVarType, UnboundType, UnionType, TypeVisitor, LiteralType,
     TypeType, NOT_READY
 )
 from mypy.visitor import NodeVisitor
+from mypy.lookup import lookup_fully_qualified
 
 
-def fixup_module_pass_one(tree: MypyFile, modules: Dict[str, MypyFile],
-                          quick_and_dirty: bool) -> None:
+# N.B: we do a quick_and_dirty fixup in both quick_and_dirty mode and
+# when fixing up a fine-grained incremental cache load (since there may
+# be cross-refs into deleted modules)
+def fixup_module(tree: MypyFile, modules: Dict[str, MypyFile],
+                 quick_and_dirty: bool) -> None:
     node_fixer = NodeFixer(modules, quick_and_dirty)
     node_fixer.visit_symbol_table(tree.names)
 
 
-def fixup_module_pass_two(tree: MypyFile, modules: Dict[str, MypyFile],
-                          quick_and_dirty: bool) -> None:
-    compute_all_mros(tree.names, modules)
-
-
-def compute_all_mros(symtab: SymbolTable, modules: Dict[str, MypyFile]) -> None:
-    for key, value in symtab.items():
-        if value.kind in (LDEF, MDEF, GDEF) and isinstance(value.node, TypeInfo):
-            info = value.node
-            info.calculate_mro()
-            assert info.mro, "No MRO calculated for %s" % (info.fullname(),)
-            compute_all_mros(info.names, modules)
-
-
 # TODO: Fix up .info when deserializing, i.e. much earlier.
 class NodeFixer(NodeVisitor[None]):
     current_info = None  # type: Optional[TypeInfo]
@@ -67,6 +56,10 @@ class NodeFixer(NodeVisitor[None]):
                 info.declared_metaclass.accept(self.type_fixer)
             if info.metaclass_type:
                 info.metaclass_type.accept(self.type_fixer)
+            if info._mro_refs:
+                info.mro = [lookup_qualified_typeinfo(self.modules, name, self.quick_and_dirty)
+                            for name in info._mro_refs]
+                info._mro_refs = None
         finally:
             self.current_info = save_info
 
@@ -76,7 +69,7 @@ class NodeFixer(NodeVisitor[None]):
         for key, value in list(symtab.items()):
             cross_ref = value.cross_ref
             if cross_ref is not None:  # Fix up cross-reference.
-                del value.cross_ref
+                value.cross_ref = None
                 if cross_ref in self.modules:
                     value.node = self.modules[cross_ref]
                 else:
@@ -84,26 +77,17 @@ class NodeFixer(NodeVisitor[None]):
                                                      self.quick_and_dirty)
                     if stnode is not None:
                         value.node = stnode.node
-                        value.type_override = stnode.type_override
-                        if (self.quick_and_dirty and value.kind == TYPE_ALIAS and
-                                stnode.type_override is None):
-                            value.type_override = Instance(stale_info(), [])
-                        value.alias_tvars = stnode.alias_tvars or []
                     elif not self.quick_and_dirty:
                         assert stnode is not None, "Could not find cross-ref %s" % (cross_ref,)
                     else:
                         # We have a missing crossref in quick mode, need to put something
-                        value.node = stale_info()
-                        if value.kind == TYPE_ALIAS:
-                            value.type_override = Instance(stale_info(), [])
+                        value.node = stale_info(self.modules)
             else:
                 if isinstance(value.node, TypeInfo):
                     # TypeInfo has no accept().  TODO: Add it?
                     self.visit_type_info(value.node)
                 elif value.node is not None:
                     value.node.accept(self)
-                if value.type_override is not None:
-                    value.type_override.accept(self.type_fixer)
 
     def visit_func_def(self, func: FuncDef) -> None:
         if self.current_info is not None:
@@ -148,6 +132,9 @@ class NodeFixer(NodeVisitor[None]):
         if v.type is not None:
             v.type.accept(self.type_fixer)
 
+    def visit_type_alias(self, a: TypeAlias) -> None:
+        a.target.accept(self.type_fixer)
+
 
 class TypeFixer(TypeVisitor[None]):
     def __init__(self, modules: Dict[str, MypyFile], quick_and_dirty: bool) -> None:
@@ -159,21 +146,17 @@ class TypeFixer(TypeVisitor[None]):
         type_ref = inst.type_ref
         if type_ref is None:
             return  # We've already been here.
-        del inst.type_ref
-        node = lookup_qualified(self.modules, type_ref, self.quick_and_dirty)
-        if isinstance(node, TypeInfo):
-            inst.type = node
-            # TODO: Is this needed or redundant?
-            # Also fix up the bases, just in case.
-            for base in inst.type.bases:
-                if base.type is NOT_READY:
-                    base.accept(self)
-        else:
-            # Looks like a missing TypeInfo in quick mode, put something there
-            assert self.quick_and_dirty, "Should never get here in normal mode"
-            inst.type = stale_info()
+        inst.type_ref = None
+        inst.type = lookup_qualified_typeinfo(self.modules, type_ref, self.quick_and_dirty)
+        # TODO: Is this needed or redundant?
+        # Also fix up the bases, just in case.
+        for base in inst.type.bases:
+            if base.type is NOT_READY:
+                base.accept(self)
         for a in inst.args:
             a.accept(self)
+        if inst.final_value is not None:
+            inst.final_value.accept(self)
 
     def visit_any(self, o: Any) -> None:
         pass  # Nothing to descend into.
@@ -226,6 +209,9 @@ class TypeFixer(TypeVisitor[None]):
         if tdt.fallback is not None:
             tdt.fallback.accept(self)
 
+    def visit_literal_type(self, lt: LiteralType) -> None:
+        lt.fallback.accept(self)
+
     def visit_type_var(self, tvt: TypeVarType) -> None:
         if tvt.values:
             for vt in tvt.values:
@@ -249,6 +235,20 @@ class TypeFixer(TypeVisitor[None]):
         t.item.accept(self)
 
 
+def lookup_qualified_typeinfo(modules: Dict[str, MypyFile], name: str,
+                              quick_and_dirty: bool) -> TypeInfo:
+    node = lookup_qualified(modules, name, quick_and_dirty)
+    if isinstance(node, TypeInfo):
+        return node
+    else:
+        # Looks like a missing TypeInfo in quick mode, put something there
+        assert quick_and_dirty, "Should never get here in normal mode," \
+                                " got {}:{} instead of TypeInfo".format(type(node).__name__,
+                                                                        node.fullname() if node
+                                                                        else '')
+        return stale_info(modules)
+
+
 def lookup_qualified(modules: Dict[str, MypyFile], name: str,
                      quick_and_dirty: bool) -> Optional[SymbolNode]:
     stnode = lookup_qualified_stnode(modules, name, quick_and_dirty)
@@ -260,43 +260,17 @@ def lookup_qualified(modules: Dict[str, MypyFile], name: str,
 
 def lookup_qualified_stnode(modules: Dict[str, MypyFile], name: str,
                             quick_and_dirty: bool) -> Optional[SymbolTableNode]:
-    head = name
-    rest = []
-    while True:
-        if '.' not in head:
-            if not quick_and_dirty:
-                assert '.' in head, "Cannot find %s" % (name,)
-            return None
-        head, tail = head.rsplit('.', 1)
-        rest.append(tail)
-        mod = modules.get(head)
-        if mod is not None:
-            break
-    names = mod.names
-    while True:
-        if not rest:
-            if not quick_and_dirty:
-                assert rest, "Cannot find %s" % (name,)
-            return None
-        key = rest.pop()
-        if key not in names:
-            if not quick_and_dirty:
-                assert key in names, "Cannot find %s for %s" % (key, name)
-            return None
-        stnode = names[key]
-        if not rest:
-            return stnode
-        node = stnode.node
-        assert isinstance(node, TypeInfo)
-        names = node.names
+    return lookup_fully_qualified(name, modules, raise_on_missing=not quick_and_dirty)
 
 
-def stale_info() -> TypeInfo:
+def stale_info(modules: Dict[str, MypyFile]) -> TypeInfo:
     suggestion = "<stale cache: consider running mypy without --quick>"
     dummy_def = ClassDef(suggestion, Block([]))
     dummy_def.fullname = suggestion
 
     info = TypeInfo(SymbolTable(), dummy_def, "<stale>")
-    info.mro = [info]
-    info.bases = []
+    obj_type = lookup_qualified(modules, 'builtins.object', False)
+    assert isinstance(obj_type, TypeInfo)
+    info.bases = [Instance(obj_type, [])]
+    info.mro = [info, obj_type]
     return info
